/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: visibility.h,v 1.4 2002/05/08 21:24:48 jdejong Exp $ */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: visibility.h,v 1.4 2002/05/08 21:24:48 jdejong Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08 imported
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */


#ifndef _VISIBILITY_H
#define _VISIBILITY_H
/*----------------------------------------------------------------------- */
/* Visibility data FITS binary tables routines                            */
/* Partially/fully calibrated data table for optical/IR interferometer    */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"

typedef struct      /* visibility table geometry descriptor */
{
  double crval[2];  /* reference value for each axis */
  float  crpix[2];  /* reference pixels */
  float  cd[2][2];  /* cd transformation matrix */
  int  naxis[2];    /* dimension of two axes of data, weight arrays */
  char *date_obs;   /* observing date */
  char *ctype[2];   /* labels for the two axes */
} visibility_geom;

typedef struct      /* table definition for visibility table */
{
  table_def *def;      /* basic table information */
  visibility_geom *geom; /* descriptor for this table */
  long   row;          /* last row (0-rel) read or written, -1=> none. */
  int    time_col;     /* time column number (0-rel)*/
  int    timei_col;    /* averaging time column number */
  int    source_col;   /* source number column number */
  int    tele_col;     /* telescope numbers column number */
  int    uvw_col;      /* uvw column number */
  int    offset_col;   /* offset column number */
  int    rotation_col; /* rotation column number */
  int    data_col;     /* data column numbers */
  int    weight_col;   /* data weight column numbers */
} visibility_table_def;

typedef struct      /* structure used for visibility row */
{
  double time;       /* central time of exposure */
  double uvw[3];     /* Spatial frequency coordinated for observations */
  float  timei;      /* data averaging time */
  float  offset[2];  /* position offset from source position */
  float  rotation;   /* rotation angle on sky */
  int    source;     /* source id */
  int    tele[2];    /* telescope numbers of array */
  float  *data;      /* pointer to visibility data in raw table row */
  float  *weight;    /* pointer to visibility weights in raw table row */
  table_row *raw;    /* generic table row */
} visibility_row;

/* public functions */

/* visibility_geom functions */
visibility_geom* 
make_visibility_geom(int  naxis[2], char *ctype[2], double crval[2], 
		     float crpix[2], float cd[2][2], char *date_obs);
/*----------------------------------------------------------------------- */
/*   Make an visibility_geom structure                                    */
/*   Inputs:                                                              */
/*      naxis      dimension of two axes of data, weight arrays           */
/*      ctype      labels for the axes                                    */
/*      crval      reference value for each axis                          */
/*      crpix      reference pixels                                       */
/*      cd         cd transformation matrix                               */
/*      date_obs    Date (2002-05-08) observations began                  */
/*   Returns:                                                             */
/*      An visibility_geom structure pointer ; NULL indicates failure     */
/*----------------------------------------------------------------------- */

void kill_visibility_geom (visibility_geom *geom);
/*----------------------------------------------------------------------- */
/*   Delete an visibility_geom structure                                  */
/*   Inputs:                                                              */
/*      geom    pointer to the visibility_geom structure.                 */
/*----------------------------------------------------------------------- */

/* visibility_row functions */
visibility_row* make_visibility_row(visibility_table_def* table);
/*----------------------------------------------------------------------- */
/*   Make an visibility_row structure                                   */
/*   Inputs:                                                              */
/*      table     pointer to visibility table structure                 */
/*   Returns:                                                             */
/*      An visibility_row structure pointer ; NULL indicates failure    */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */

void kill_visibility_row (visibility_row *row);
/*----------------------------------------------------------------------- */
/*   Delete an visibility_row structure                                   */
/*   Inputs:                                                              */
/*      row    pointer to the visibility_row structure.                   */
/*----------------------------------------------------------------------- */

/* visibility_table functions */
visibility_table_def* 
make_visibility_table_def(fitsfile *fptr, visibility_geom *geom,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Make a new visibility_table in an open FITS file                     */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      geom  visibility_geom_table describing table                      */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An visibility_table_def pointer; NULL indicates failure           */
/*----------------------------------------------------------------------- */

visibility_table_def* 
open_visibility_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Open an existing visibility_table in the active FITS file            */
/*   Fills visibility_geom and saves in visibility_table_def              */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An visibility_table_def pointer; NULL indicates failure           */
/*----------------------------------------------------------------------- */

int read_visibility_table(fitsfile *fptr,  visibility_table_def* table, 
			    visibility_row *row, long row_number,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Read the next row from an visibility_table                           */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int write_visibility_table(fitsfile *fptr, visibility_table_def* table, 
			     visibility_row *row,  long row_number,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write  the next row to an visibility_table                         */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */     
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int close_visibility_table(fitsfile *fptr, visibility_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Close an open visibility_table, associated structures are destroyed  */
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

void kill_visibility_table_def (visibility_table_def* table);
/*----------------------------------------------------------------------- */
/*   Delete an visibility_table structure                                 */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */

#endif /* end VISIBILITY_H */
/* */
